### Figure 3.2
source('bookfunctions.r')
library(geoR)
spatialdata = read.csv(file = 'Wind Spatial Dataset.csv', header = T)
locationdata = spatialdata[1:2,]
latitude = t(locationdata[1,-1])
longitude = t(locationdata[2,-1])
labels = c('Turbine #1','Turbine #2','Turbine #3','Turbine #4','Turbine #5','Turbine #6','Turbine #7','Turbine #8','Turbine #9','Turbine #10')
pdf('turbinelocation.pdf')
plot(x = longitude, y = latitude, type = 'p', pch = 17, xlab = 'Longitude', ylab = 'Latitude', cex.lab = 1.33, cex.axis = 1.33)
dev.off()
wsdata = spatialdata[4:nrow(spatialdata),]
colnames(wsdata) = c('Time',labels)
train.month = '^7/'
test.month = '^8/'
traindata = wsdata[grep(train.month,wsdata$Time),]
traindata = t(traindata[,-1])
train.time.series = NULL
for (i in 1:ncol(traindata)){
  for (j in 1:nrow(traindata)){
    train.time.series = rbind(train.time.series,traindata[j,i])
  }
}
testdata = wsdata[grep(test.month,wsdata$Time),]
testdata = t(testdata[,-1])
test_x = testdata[-6,]
test_y = testdata[6,]
coords = data.frame(longitude,latitude)
test_x.coords = coords[-6,]
test_y.coords = coords[6,]
V_pred = NULL
coords.ts = NULL
for (i in 1:ncol(traindata)){
  for (j in 1:nrow(traindata)){
    coords.ts = rbind(coords.ts,coords[j,])
  }
}
params = likfit(coords = coords.ts, data = train.time.series, ini.cov.pars = c(1,1), nugget = 0.5)
for (i in 1:ncol(test_x)){
  V_pred[i] = krige.conv(coords = test_x.coords, data = test_x[,i], locations = test_y.coords, krige = krige.control(obj.model = params))$predict
}
pdf('kriging.pdf')
matplot(y = data.frame(test_y,V_pred), type = 'o', pch = 19, cex = 0.5, col = c('blue','red'), lwd = 1, ylab = "Wind speed (m/s)", xlab = 'Time (hours)', cex.lab = 1.33, cex.axis  = 1.33)
legend('topleft', legend = c('Observed wind speed','Predicted wind speed'), lty = c(1,2), col = c('blue','red'))
dev.off()
pdf('kriging10days.pdf')
matplot(y = data.frame(test_y[1:240],V_pred[1:240]), type = 'o', pch = 19, cex = 0.5, col = c('blue','red'), lwd = 1, ylab = "Wind speed (m/s)", xlab = 'Time (hours)', cex.lab = 1.33, cex.axis  = 1.33)
legend('topleft', legend = c('Observed wind speed','Predicted wind speed'), lty = c(1,2), col = c('blue','red'))
dev.off()
pdf('kriging1week.pdf')
matplot(y = data.frame(test_y[1:168],V_pred[1:168]), type = 'o', pch = 19, cex = 0.5, col = c('blue','red'), lwd = 1, ylab = "Wind speed (m/s)", xlab = 'Time (hours)', cex.lab = 1.33, cex.axis  = 1.33)
legend('topleft', legend = c('Observed wind speed','Predicted wind speed'), lty = c(1,2), col = c('blue','red'))
dev.off()
