################################################################################
#File Description: 
#This file generates Figure 3.4 in Chapter 3 of DSWE
#Computing asymmetry for one year of data based on yearly average wind direction
################################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(circular)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly
#Functions to import
source("Code Used in the Book/Chapter3/funcs/harmonics_fit.R")
source("Code Used in the Book/Chapter3/funcs/asym_compute.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
kvalues = seq(1,24,1) #Lags at which to compute asymmetry


#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
yearlydata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##Mast data
mastdata = yearlydata[5:(Period+4),c(406,407)] #Data from Mast 3 - used throughout for this analysis
colnames(mastdata) = c("Speed","Direction")

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1) #Index of Turbines
coords[,2] = as.numeric(yearlydata[1,2:201]) #Latitude
coords[,3] = as.numeric(yearlydata[2,2:201]) #Longitude
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

##Turbine-specific wind speeds and time indices
nn = N * Period #number of spatio-temporal data instances for the whole year
turb.data = matrix(nrow = nn,ncol=2) #matrix containing spatio-temporal wind speeds, and diurnal time index
aa = bb = 1
cat("PREPARING DATA FOR ANALYSIS - MAY TAKE COUPLE OF MINUTES","\n")
for(ii in 1:Period){
  turb.data[aa:(aa+N-1),1] = as.numeric(yearlydata[ii+4,seq(2,401,2)])
  turb.data[aa:(aa+N-1),2] = rep(bb,N)
  aa = aa + N
  bb = bb + 1
  if(bb > 24){bb = 1}
}
colnames(turb.data) = c("Speed","t")

#***********************************************************************
#****************ASYMMETRY COMPUTATION**********************************
#***********************************************************************
r = harmonics.fit(turb.data)
out.list = asym.compute(rvec=r,dirdata=as.numeric(mastdata[,2]),locs=coords,kvals=kvalues,per=Period,st=1,end=Period)
asym_list = out.list[[1]]
dist_list = out.list[[2]]

#Plot1: Yearly analysis versus time lag
par(cex.main = 1.5,mar=c(5,7,4,4)+0.1,cex.lab=1.2,font.lab=1.2,cex.axis=1.2,las=1)
kvals.id = seq(0,24,1)
plot(seq(1,length(kvals.id),1), rep(-10,length(kvals.id)), type = "p", ylab = " ", font.lab = 2,xlab = "Time lag (hours)", cex.lab = 1.75, 
     ylim = c(-0.05, 0.1), xlim = c(1, length(kvals.id)+1), lwd = 3, pch = 5, axes = F, main = " ",panel.first=grid())
axis(1, at = seq(1,length(kvals.id),3),labels = seq(0,24,3),cex.axis=1.75,cex.lab=1.75,lwd=3)
axis(2, pos =, seq(-0.05,0.1,0.05),cex.lab=1.75,cex.axis=1.75,lwd=3)
par(las = 0)
box(lwd=4,col="black")
mtext("Asymmetry in Correlation", side = 2, line = 4, cex = 1.75, font = 2,lwd=3)

kvals.id = seq(0,24,3)
box = boxplot(0,asym_list[[3]],asym_list[[6]],asym_list[[9]],asym_list[[12]],asym_list[[15]],asym_list[[18]],asym_list[[21]],asym_list[[24]],plot=F,
              outline=F,axes=F,add=T,out=NULL,col="grey80",frame.plot=F,border="darkblue",boxwex = 0.6,lwd=3)
boxplot(0,asym_list[[3]],asym_list[[6]],asym_list[[9]],asym_list[[12]],asym_list[[15]],asym_list[[18]],asym_list[[21]],asym_list[[24]],
        add=T,at = kvals.id+1 ,range=10^-5,outline=F,axes=F,
        boxlty=0,border=c("darkblue"),boxwex=0.6)

matt = matrix(ncol=length(kvals.id),nrow=3)
matt[1,]= box$stats[2,];matt[2,]=box$stats[3,];matt[3,]=box$stats[4,]
for (i in 1:length(kvals.id)){
  segments(x0=kvals.id[i]+1,y0=matt[1,i]+0.0006,x1=kvals.id[i]+1,y1=matt[3,i]-0.0006,col="darkblue",lwd=4)
  points(kvals.id[i]+1,matt[2,i],pch=19,col="darkred",lwd=3,cex=2.5)
}
segments(x1=13,y0=box$stats[3,5],x0=-1,y1=box$stats[3,5],lwd=3,lty=4,col="black")
text(x=3.5,y=0.03,cex=1.4,expression(paste(italic(bar("A")),"(",bold("s"),",",italic("h*"),")"," = 0.024")))
segments(x0=13,y0=-1,x1=13,y1=box$stats[3,5],lwd=3,lty=4,col="black")
text(x=15,y=-0.04,cex=1.4,expression(paste(italic("h*")," = 12")))

#***********************************************************************
#***************************SAVE IMPORTANT FILES************************
#***********************************************************************
saveRDS(asym_list, "Code Used in the Book/Chapter3/temp/asym_year.rds")  
saveRDS(dist_list, "Code Used in the Book/Chapter3/temp/dist_year.rds") 

