################################################################################
#File Description: 
#This file generates Figure 3.5 in Chapter 3 of DSWE
#Change Point Detection for the Wind Direction (circular) Variable
################################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(circular)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
nsims = 1000 #number of MC simulations to determine cutoff for the test
cutoff = 0.95 #cutoff quantile

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
yearlydata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours

##Mast data
mastdata = yearlydata[5:(Period+4),c(406,407)] #Data from Mast 3 - used throughout for this analysis
colnames(mastdata) = c("Speed","Direction")
head(mastdata)
dir = as.numeric(mastdata$Direction)

#***********************************************************************
#*****************CIRCULAR CHANGE POINT DETECTION***********************
#***********************************************************************
#Fit a Von Mises Distribution for the Wind Direction
x = circular(dir,units="degrees")
mle = mle.vonmises(x)
kappa.hat = mle$kappa #MLE estimate for the concentration parameter 

#Change Point Detection
x_all = circular(dir,units="degrees")
LL = 1 #lower limit on prevailing period (1 hour)
sliding = 1
begin = 1
id_overall  = list()
cat("SEARCHING FOR CPs","\n")
repeat{
    range = begin:(begin + 24*7*4 - 1) #1 month rolling binary segmentation
    if ( begin > 8755){break}
    options(warn=-1)
    x = x_all[range]
    dir_subset = dir[range]
    indices = range
    kappa.hat = mle.vonmises(x)$kappa
    rmax = vector(length = nsims)
    for (i in 1:nsims){ #MC simulations to determine distribution of cut off to get the 5 or 1 % cut-off
      set.seed(i)
      simulated_data = rvonmises(n=length(x),mu=circular(0),kappa=kappa.hat)
      rmax[i] = change.point(simulated_data)$rmax
    }
    cut_level = cutoff
    cut_val = quantile(rmax,cut_level)
  
  #Binary Segmentation code
  x.id = seq(1,length(x),1)
  change = function(s,e){ #function for change point detection that renders the rmax value and the index of candidate change point
    sub = x[s:e]
    temppp = mean.circular(x[s:e],na.rm=T)
    if (temppp < 0){temppp = temppp + 360}
    sub[which(is.na(sub) == T)] = temppp
    detection = change.point(sub)
    index = s + detection$k.r - 1
    out = c(detection$rmax,index)
    return(out)
  }
  
  id = list()
  rlist = list()
  iter = 1
  start = 1
  end = length(x)
  num_partitions = 1
  limits = c(start,end)
  repeat{
    part = 1
    vec = vector(length = 1)
    vec2 = vector(length = 1)
    sig = 0
    for (part in 1:num_partitions){ 
      start = limits[part] + ifelse(part > 1,1,0)
      end = limits[part + 1]
      cp = change(start,end)
      r = cp[1]
      r
      if (r >= cut_val){ #significant change point
        sig = sig + 1
      vec[sig] = cp[2]
      vec2[sig] = r
      }}
    
    if(sig == 0){ #didnt find any change in any partition
      break
    }
    
    id[[iter]] = vec
    rlist[[iter]] = vec2
    num_partitions = length(unlist(id)) + 1
    limits = c(1,sort(unlist(id)),length(x))
    iter = iter + 1
  }
  
  ids = unlist(id)
  ids = begin + ids - 1
  id_overall[[sliding]] = ids
  
  
  if(length(ids) != 0){
  begin = max(ids) + 1}else{begin=begin + 24*7*4 - 1}
  
  number_cps = length(unlist(id_overall))
  differences = c(dist(ids))
  flush.console()
  cat("Total number of CPs detected so far"," ",number_cps,"\n")
  cat(begin," of 8760 data points searched","\n")
  
  if (min(differences) < LL ){break}

  sliding = sliding + 1
}

#***********************************************************************
#*********************PLOTTING AND SAVING RESULTS***********************
#***********************************************************************
changepoints = sort(unlist(id_overall))
par(mfrow=c(1,1),mar=c(5,6,2,2))
start = 1
range = start : (start + (24*7*2) - 1)
subset = dir[range]
plot(subset,xaxt="n",yaxt="n",xlab="Time (days)",cex=1.5,font.lab=2,panel.first=grid(),pch="*",ylab="Direction (degs)",ylim=c(0,360),cex.lab=1.75,cex.axis=1.75,lwd=2)
axis(1,seq(1,(24*7*2 + (24*1)),48),seq(0,14,2),lwd=2,cex.axis=1.75,cex.lab=1.75)
axis(2,seq(0,360,90),lwd=2,cex.axis=1.75,cex.lab=1.75)
cps = changepoints[which(changepoints >= range[1] & changepoints <= tail(range,1))]
new.cps = cps - range[1] + 1
for (j in 1:length(new.cps)){
  points(new.cps[j],subset[new.cps[j]],col="red",pch=0,cex=2)
  abline(v = new.cps[j],lty=6,col="red",lwd=2)
}
box(lwd=4)

# Length of PPs in hours
pp.length = vector(length = length(changepoints) + 1)
pp.length[1] = changepoints[1] - 1
for (i in 2:length(changepoints)){
   pp.length[i] = changepoints[i] - changepoints[i-1]}
pp.length[i+1] = Period - changepoints[i]

##Save changepoints and lengths of prevailing periods for later analysis
changepoints = sort(unlist(id_overall))
write.table(changepoints,"Code Used in the Book/Chapter3/temp/changepoints.txt")
write.table(pp.length,"Code Used in the Book/Chapter3/temp/pp_length.txt")

