################################################################################
#File Description: 
#This file generates Figure 3.6 in Chapter 3 of DSWE
#Computing asymmetry for yearly, seasonal, monthly, weekly, T-lens, ST-lens scenarios
################################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(circular)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter3/funcs/harmonics_fit.R")
source("Code Used in the Book/Chapter3/funcs/asym_compute.R")
source("Code Used in the Book/Chapter3/funcs/asym_compute_st.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
kvalues = seq(1,24,1) #Time lags at which to compute asymmetry

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
yearlydata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##Mast data
mastdata = yearlydata[5:(Period+4),c(406,407)] #Data from Mast 3 - used throughout for this analysis
colnames(mastdata) = c("Speed","Direction")

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1) #Index of Turbines
coords[,2] = as.numeric(yearlydata[1,2:201]) #Latitude
coords[,3] = as.numeric(yearlydata[2,2:201]) #Longitude
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#***********************************************************************
#************ASYMMETRY COMPUTATION - YEARLY SCENARIO********************
#***********************************************************************

##Results imported after compiling the file Code_(Figure3.4).R first
asym_year = readRDS("Code Used in the Book/Chapter3/temp/asym_year.rds")
dist_year = readRDS("Code Used in the Book/Chapter3/temp/dist_year.rds")

#***********************************************************************
#************ASYMMETRY COMPUTATION - SEASONAL SCENARIO******************
#***********************************************************************
num_seasons = 4
##Separating data into seasons
fall_data = yearlydata[5:2189,]
winter_data = yearlydata[2190:4349,]
spring_data = yearlydata[4350:6557,]
summer_data = yearlydata[6558:Period,]

##Turbine-specific wind speeds and time indices
fall_dat = matrix(nrow=dim(fall_data)[1]*N,ncol=2)
aa = bb = 1
for(ii in 1:dim(fall_data)[1]){
  fall_dat[aa:(aa+N-1),1] = as.numeric(fall_data[ii,seq(2,401,2)])
  fall_dat[aa:(aa+N-1),2] = rep(bb,N)
  aa = aa + N
  bb = bb + 1
  if(bb > 24){bb = 1}
}
winter_dat = matrix(nrow=dim(winter_data)[1]*N,ncol=2)
aa = bb = 1
for(ii in 1:dim(winter_data)[1]){
  winter_dat[aa:(aa+N-1),1] = as.numeric(winter_data[ii,seq(2,401,2)])
  winter_dat[aa:(aa+N-1),2] = rep(bb,N)
  aa = aa + N
  bb = bb + 1
  if(bb > 24){bb = 1}
}
spring_dat = matrix(nrow=dim(spring_data)[1]*N,ncol=2)
aa = bb = 1
for(ii in 1:dim(spring_data)[1]){
  spring_dat[aa:(aa+N-1),1] = as.numeric(spring_data[ii,seq(2,401,2)])
  spring_dat[aa:(aa+N-1),2] = rep(bb,N)
  aa = aa + N
  bb = bb + 1
  if(bb > 24){bb = 1}
}
summer_dat = matrix(nrow=dim(summer_data)[1]*N,ncol=2)
aa = bb = 1
for(ii in 1:dim(summer_data)[1]){
  summer_dat[aa:(aa+N-1),1] = as.numeric(summer_data[ii,seq(2,401,2)])
  summer_dat[aa:(aa+N-1),2] = rep(bb,N)
  aa = aa + N
  bb = bb + 1
  if(bb > 24){bb = 1}
}

asym_seasons = list(length = num_seasons)
dist_seasons = list(length = num_seasons)
for (seasons in 1:num_seasons){
  if (seasons == 1){datas = fall_dat;st=1;end=2185}
  if(seasons == 2){datas = winter_dat;st=2186;end=4345}
  if (seasons ==3){datas = spring_dat;st=4346;end=6553}
  if (seasons == 4){datas = summer_dat;st=6554;end=8760}
  Period = dim(datas)[1]/200
  nn = N * Period #number of spatio-temporal data instances
  
  r = harmonics.fit(datas)
  
  out.list = asym.compute(rvec=r,dirdata=as.numeric(mastdata[,2]),locs=coords,kvals=kvalues,per=Period,st=st,end=end)
  asym_list = out.list[[1]]
  dist_list = out.list[[2]] 

  asym_seasons[[seasons]] = asym_list
  dist_seasons[[seasons]] = dist_list
  cat(paste("SEASON # ",seasons,"out of 4 seasons",sep=" "),"\n")
}

#Find kstar for every season
kstar_seasons = vector(length = num_seasons)
for (j in 1:num_seasons){
  medians = vector(length = length(kvalues))
  for (i in 1:length(kvalues)){
    medians[i] = median(asym_seasons[[j]][[i]])
  }
  kstar_seasons[j] =   which(medians == max(medians))
}

#save important files
saveRDS(asym_seasons, "Code Used in the Book/Chapter3/temp/asym_seasons.rds")  
saveRDS(dist_seasons, "Code Used in the Book/Chapter3/temp/dist_seasons.rds") 
write.table(as.numeric(kstar_seasons),file="Code Used in the Book/Chapter3/temp/kstar_seasons.txt")

################################################################################
#************ASYMMETRY COMPUTATION - MONTHLY SCENARIO***************************
################################################################################
num_months = 12
numdayspermonth = c(30,31,30,31,31,28,31,30,31,30,31,31)
months_data = list(length = 12)
aa = 1
dats = yearlydata[5:8764,]
for(ii in 1:num_months){
  months_data[[ii]] = dats[aa:(aa-1+numdayspermonth[ii]*24),]
  aa = aa + numdayspermonth[ii]*24
}

##Restructuring data
months_dat = list(length = num_months)
for(i in 1:num_months){
  mm_raw = months_data[[i]]
  mm = matrix(nrow=dim(mm_raw)[1]*N,ncol=2)
  aa = bb = 1
  for(ii in 1:dim(mm_raw)[1]){
    mm[aa:(aa+N-1),1] = as.numeric(mm_raw[ii,seq(2,401,2)])
    mm[aa:(aa+N-1),2] = rep(bb,N)
    aa = aa + N
    bb = bb + 1
    if(bb > 24){bb = 1}
  }
  months_dat[[i]] = mm
}

asym_months = list(length = num_months)
dist_months = list(length = num_months)
st = 1;end = numdayspermonth[1]*24
for (months in 1:num_months){
  datas = months_dat[[months]]
  Period = dim(datas)[1]/200
  nn = N * Period #number of spatio-temporal data instances
  r = harmonics.fit(datas)
  out.list = asym.compute(rvec=r,dirdata=as.numeric(mastdata[,2]),locs=coords,kvals=seq(1,24,1),per=Period,st=st,end=end)
  asym_list = out.list[[1]]
  dist_list = out.list[[2]] 
  asym_months[[months]] = asym_list
  dist_months[[months]] = dist_list
  cat("MONTH # ",months,"\n")
  st = end + 1
  end = st + numdayspermonth[months]*24 - 1
}

#Find kstar for every season
kstar_months = vector(length = num_months)
for (j in 1:num_months){
  medians = vector(length = length(kvalues))
  for (i in 1:length(kvalues)){
    medians[i] = median(asym_months[[j]][[i]])
  }
  kstar_months[j] =   which(medians == max(medians))
}

#save important files
saveRDS(asym_months, "Code Used in the Book/Chapter3/temp/asym_months.rds")  
saveRDS(dist_months, "Code Used in the Book/Chapter3/temp/dist_months.rds") 
write.table(as.numeric(kstar_months),file="Code Used in the Book/Chapter3/temp/kstar_months.txt")

################################################################################
#************ASYMMETRY COMPUTATION - WEEKLY SCENARIO****************************
################################################################################
##Separating data into months
num_weeks = 52
numdaysperweek = 7
weeks_data = list(length = num_weeks)
aa = 1
dats = yearlydata[5:8764,]
for(ii in 1:num_weeks){
  weeks_data[[ii]] = dats[aa:(aa-1+numdaysperweek*24),]
  aa = aa + numdaysperweek*24
}

##Restructuring data
weeks_dat = list(length = num_weeks)
for(i in 1:num_weeks){
  ww_raw = weeks_data[[i]]
  ww = matrix(nrow=dim(ww_raw)[1]*N,ncol=2)
  aa = bb = 1
  for(ii in 1:dim(ww_raw)[1]){
    ww[aa:(aa+N-1),1] = as.numeric(ww_raw[ii,seq(2,401,2)])
    ww[aa:(aa+N-1),2] = rep(bb,N)
    aa = aa + N
    bb = bb + 1
    if(bb > 24){bb = 1}
  }
  weeks_dat[[i]] = mm
  cat(i,"out of 52","\r")
}

asym_weeks = list(length = num_weeks)
dist_weeks = list(length = num_weeks)
st = 1;end = numdaysperweek*24
for (weeks in 1:num_weeks){
  datas = weeks_dat[[weeks]]
  Period = dim(datas)[1]/200
  nn = N * Period #number of spatio-temporal data instances
  r = harmonics.fit(datas)
  out.list = asym.compute(rvec=r,dirdata=as.numeric(mastdata[,2]),locs=coords,kvals=seq(1,24,1),per=Period,st=st,end=end)
  asym_list = out.list[[1]]
  dist_list = out.list[[2]] 
  asym_weeks[[weeks]] = asym_list
  dist_weeks[[weeks]] = dist_list
  cat("WEEK # ",weeks,"\n")
  st = end + 1
  end = st + numdaysperweek*24 - 1
}

#Find kstar for every week
kstar_weeks = vector(length = num_weeks)
for (j in 1:num_weeks){
  medians = vector(length = 24)
  for (i in 1:24){
    medians[i] = median(asym_weeks[[j]][[i]])
  }
  kstar_weeks[j] =   which(medians == max(medians))
}

#save important files
saveRDS(asym_weeks, "Code Used in the Book/Chapter3/temp/asym_weeks.rds")  
saveRDS(dist_weeks, "Code Used in the Book/Chapter3/temp/dist_weeks.rds") 
write.table(kstar_weeks,file="Code Used in the Book/Chapter3/temp/kstar_weeks.txt")

################################################################################
#************ASYMMETRY COMPUTATION - TEMPORAL LENS SCENARIO*********************
################################################################################
ids_PP= as.numeric(unlist(read.table("Code Used in the Book/Chapter3/temp/changepoints.txt")))
pp_len = as.numeric(unlist(read.table("Code Used in the Book/Chapter3/temp/pp_length.txt")))
asym_PP = dist_PP = list(length = (length(ids_PP)))

for (pp in 1:(length(ids_PP))){
  if (pp == 1){st=1;end=ids_PP[pp]}
  if (pp > 1){st=ids_PP[pp-1]+1;end=ids_PP[pp]}
  datas = yearlydata[(st+4):(end+4),]
  Period = dim(datas)[1]
  nn = N * Period #number of spatio-temporal data instances
  
  kvalues = seq(1,24,1) #Time lags at which to compute asymmetry
  if (pp_len[pp] < 26){ 
    sel = which(kvalues < (pp_len[pp]-2))
    kvalues = kvalues[sel]
  }
  
  turb.data = matrix(nrow = nn,ncol=2) #matrix containing spatio-temporal wind speeds, and diurnal time index
  aa = bb = 1
  for(ii in 1:Period){
    turb.data[aa:(aa+N-1),1] = as.numeric(datas[ii,seq(2,401,2)])
    turb.data[aa:(aa+N-1),2] = rep(bb,N)
    aa = aa + N
    bb = bb + 1
    if(bb > 24){bb = 1}
  }
  colnames(turb.data) = c("Speed","t")
  r = harmonics.fit(turb.data)
  out.list = asym.compute(rvec=r,dirdata=as.numeric(mastdata[,2]),locs=coords,kvals=kvalues,per=Period,st=st,end=end)
  asym_list = out.list[[1]]
  dist_list = out.list[[2]] 
  asym_PP[[pp]] = asym_list
  dist_PP[[pp]] = dist_list
  cat("PP # ",pp,"\n")
}

#Find kstar for every PP
kstar_pp = vector(length = length(ids_PP))
for (j in 1:length(kstar_pp)){
  medians = vector(length = length(asym_PP[[j]]))
  for (i in 1:length(asym_PP[[j]])){
    medians[i] = median(asym_PP[[j]][[i]])
  }
  kstar_pp[j] = which(medians == max(medians))[1]
}

#save important files
saveRDS(asym_PP, "Code Used in the Book/Chapter3/temp/asym_PP.rds")  
saveRDS(dist_PP, "Code Used in the Book/Chapter3/temp/dist_PP.rds") 
write.table(kstar_pp,file="Code Used in the Book/Chapter3/temp/kstar_pp.txt")

################################################################################
#************ASYMMETRY COMPUTATION - SPATIO-TEMPORAL LENS SCENARIO**************
################################################################################
asym_PPBW = dist_PPBW = list(length = (length(ids_PP)))
bands = seq(5,90,5)/2

for (pp in 1:(length(ids_PP))){
  if (pp == 1){st=1;end=ids_PP[pp]}
  if (pp > 1){st=ids_PP[pp-1]+1;end=ids_PP[pp]}
  datas = yearlydata[(st+4):(end+4),]
  Period = dim(datas)[1]
  nn = N * Period #number of spatio-temporal data instances
  
  turb.data = matrix(nrow = nn,ncol=2) #matrix containing spatio-temporal wind speeds, and diurnal time index
  aa = bb = 1
  for(ii in 1:Period){
    turb.data[aa:(aa+N-1),1] = as.numeric(datas[ii,seq(2,401,2)])
    turb.data[aa:(aa+N-1),2] = rep(bb,N)
    aa = aa + N
    bb = bb + 1
    if(bb > 24){bb = 1}
  }
  colnames(turb.data) = c("Speed","t")
  r = harmonics.fit(turb.data)
  
  kval = kstar_pp[pp]
  
  out.list = asym.compute.st(rvec=r,dirdata=as.numeric(mastdata[,2]),locs=coords,kvals=kval,per=Period,st=st,end=end,bands=bands)
  asym_list = out.list[[1]]
  dist_list = out.list[[2]] 
  
  asym_PPBW[[pp]] = asym_list
  dist_PPBW[[pp]] = dist_list
  cat("PP # ",pp,"\n")
}

#Find bwstar for every PPBW
bwstar_pp = vector(length = length(ids_PP))
bands = seq(5,90,5)/2
for (j in 1:length(bwstar_pp)){
  medians = vector(length = length(asym_PPBW[[j]]))
  for (i in 1:length(asym_PPBW[[j]])){
    medians[i] = median(asym_PPBW[[j]][[i]])
  }
  bwstar_pp[j] = bands[which(medians == max(medians))[1]]
}
saveRDS(asym_PPBW, "Code Used in the Book/Chapter3/temp/asym_PPBW.rds")  
saveRDS(dist_PPBW, "Code Used in the Book/Chapter3/temp/dist_PPBW.rds") 
write.table(as.numeric(bwstar_pp),"Code Used in the Book/Chapter3/temp/bwstar_pp.txt")



################################################################################
#************************FINAL PLOTTING AND RESULTS*****************************
################################################################################
##Aggregation
#Seasonal aggregation at k_i^* 
tempp = tempp_d = 0
for (j in 1:num_seasons){
  tempp = c(tempp,asym_seasons[[j]][[kstar_seasons[j]]])
  tempp_d = c(tempp_d,dist_seasons[[j]][[kstar_seasons[j]]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_season = as.matrix(cbind(tempp_d,tempp))
#Monthly aggregation at k_i^* 
tempp = tempp_d = 0
for (j in 1:12){
  tempp = c(tempp,asym_months[[j]][[kstar_months[j]]])
  tempp_d = c(tempp_d,dist_months[[j]][[kstar_months[j]]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_month = as.matrix(cbind(tempp_d,tempp))
#Weekly aggregation at k_i^* 
tempp = tempp_d = 0
for (j in 1:num_weeks){
  tempp = c(tempp,asym_weeks[[j]][[kstar_weeks[j]]])
  tempp_d = c(tempp_d,dist_weeks[[j]][[kstar_weeks[j]]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_week = as.matrix(cbind(tempp_d,tempp))
#PP aggregation at k_i^* 
tempp = tempp_d = 0
for (j in 1:length(ids_PP)){
  tempp = c(tempp,asym_PP[[j]][[kstar_pp[j]]])
  tempp_d = c(tempp_d,dist_PP[[j]][[kstar_pp[j]]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_pp = as.matrix(cbind(tempp_d,tempp))
#PPBW aggregation at k_i^* and bw_i^* 
tempp = tempp_d = 0
bwstar_id = vector(length = length(ids_PP))
for (iii in 1:length(bwstar_pp)){
  bwstar_id[iii] = which(bands == bwstar_pp[iii])
}
for (j in 1:length(ids_PP)){
  tempp = c(tempp,asym_PPBW[[j]][[bwstar_id[j]]])
  tempp_d = c(tempp_d,dist_PPBW[[j]][[bwstar_id[j]]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_ppbw = as.matrix(cbind(tempp_d,tempp))

#Figure 3.6
x = agg_pp[,1]
par(cex.main = 1.5,mar=c(5,7,4,4)+0.1,cex.lab=1.2,font.lab=1.2,cex.axis=1.2,las=1)
plot(x, rep(-10,length(x)), type = "p", ylab = " ", xlab = " ", cex = 1, 
     ylim = c(-0.1,1), xlim = c(0,30), lwd = 2, pch = 5, axes = F, main = " ",panel.first=grid())
par(las = 0)
axis(1, at = c(5,15,25),labels = c("0-10 Km","10-20 Km","20-30 Km"),cex.axis=1.75,cex.lab=1.75,lwd=3)
mtext("Distance (Km)", side = 1, line = 3, cex = 1.75, font = 2,lwd=3)
axis(2, pos = , seq(0,1,0.2),cex.lab=1.75,cex.axis=1.75,lwd=3)
mtext("Asymmetry in correlation", side = 2, line = 4, cex = 1.75, font = 2,lwd=3)
box(lwd=4,col="black")
bounds = c(0,10,20,30)
prob = vector(length = length(bounds)-1)
for (jj in 1:(length(bounds)-1)){
  locs = seq(2,8,len=6)
  bound = bounds[jj+1]
  g1_y = as.numeric(unlist(asym_year[[12]][which(dist_year[[12]] < bound & dist_year[[12]] >= bounds[jj])]))
  g1_s = agg_season[which(agg_season[,1] < bound & agg_season[,1] >= bounds[jj]),2]
  g1_m = agg_month[which(agg_month[,1] < bound & agg_month[,1] >= bounds[jj]),2]
  g1_w = agg_week[which(agg_week[,1] < bound & agg_week[,1] >= bounds[jj]),2]
  g1_pp = agg_pp[which(agg_pp[,1] < bound & agg_pp[,1] >= bounds[jj]),2]
  g1_ppbw = agg_ppbw[which(agg_ppbw[,1] < bound & agg_ppbw[,1] >= bounds[jj]),2]
  prob[jj] = length(which(g1_ppbw > 0))/length(g1_ppbw)
  locs = locs + bounds[jj]
  boxplot(g1_y,g1_s,g1_m,g1_w,g1_pp,g1_ppbw,add=T,at = locs ,range=10^-5,outline=F,axes=F,
          boxlty=0,border="black",boxwex=1,lwd=4,medcol="white")
  box = boxplot(g1_y,g1_s,g1_m,g1_w,g1_pp,g1_ppbw,add=T,at = locs ,range=10^-5,outline=F,axes=F,
                boxwex =0.4, boxlty=0,border=c("black","red","blue","green"),plot=F)
  cols = c("black","darkred","darkblue","darkgreen","aquamarine","darkgoldenrod1")
  pchs = c(4,15,19,17,18,25)
  bg.cols = cols
  for (ii in 1:6){
    segments(x0=locs[ii],y0=box$stats[2,ii]+0.002,x1=locs[ii],y1=box$stats[4,ii]-0.002,col="black",lwd=4)
    points(locs[ii],box$stats[3,ii],pch=pchs[ii],col=cols[ii],lwd=ifelse(ii==1,4,4),cex=ifelse(ii==1,3,3),bg=bg.cols[ii])
  }
}
legend(legend=c("Yearly","Seasonal","Monthly","Weekly","T-lens","ST-lens"),col=cols,bty="n",pch=pchs,pt.bg = bg.cols,lwd=c(2,2,2,2,2,2),"topleft",y.intersp=0.35,cex=2)

#Table 3.4
meds_pp = meds_ppbw = vector(length = length(ids_PP))
for (i in 1:length(ids_PP)){
  meds_ppbw[i] = median(asym_PPBW[[i]][[bwstar_id[i]]])
}
ppmat = matrix(nrow=5,ncol=2)
ppmat[,1] = seq(1,5,1)
ppmat[1,2]=length(which(meds_ppbw <= 0))/length(ids_PP)
ppmat[2,2]=length(which(meds_ppbw > 0 & meds_ppbw < 0.05))/length(ids_PP)
ppmat[3,2]=length(which(meds_ppbw >= 0.05 & meds_ppbw < 0.1))/length(ids_PP)
ppmat[4,2]=length(which(meds_ppbw >= 0.1 & meds_ppbw < 0.2))/length(ids_PP)
ppmat[5,2]=length(which(meds_ppbw >= 0.2))/length(ids_PP)
colnames(ppmat) = c("Group","Percentage")
write.table(ppmat,file="Table3.4.txt",col.names = T,row.names = T)

