################################################################################
#File Description: 
#This file generates Figure 3.7 in Chapter 3 of DSWE
#Computing wake effect and asymmetry
################################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(circular)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter3/funcs/harmonics_fit.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
#Spatial partition on latitude and longitude
bounds.lat = 40.462
bounds.long=-88.64

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
yearlydata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##Mast data
mastdata = yearlydata[5:(Period+4),c(406,407)] #Data from Mast 3 - used throughout for this analysis
colnames(mastdata) = c("Speed","Direction")

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1) #Index of Turbines
coords[,2] = as.numeric(yearlydata[1,2:201]) #Latitude
coords[,3] = as.numeric(yearlydata[2,2:201]) #Longitude
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#***********************************************************************
#*************************LOADING REQUIRED FILES************************
#***********************************************************************
ids_PP= as.numeric(unlist(read.table("Code Used in the Book/Chapter3/temp/changepoints.txt"))) #changepoints
pp_length = as.numeric(unlist(read.table("Code Used in the Book/Chapter3/temp/pp_length.txt"))) #pp lengths
pp_kstar = as.numeric(unlist(read.table("Code Used in the Book/Chapter3/temp/kstar_pp.txt"))) #k*
pp_band = as.numeric(unlist(read.table("Code Used in the Book/Chapter3/temp/bwstar_pp.txt"))) #bw*

#***********************************************************************
#************ASYMMETRY COMPUTATION - WAKE EFFECT ***********************
#***********************************************************************
asym_PP = list(length = length(ids_PP))
dist_PP = list(length = length(ids_PP))

for (pp in 1:(length(ids_PP))){
  if (pp == 1){st=1;end=ids_PP[pp]}
  if (pp > 1){st=ids_PP[pp-1]+1;end=ids_PP[pp]}
  datas = yearlydata[(st+4):(end+4),]
  Period = dim(datas)[1]
  nn = N * Period #number of spatio-temporal data instances
  
  turb.data = matrix(nrow = nn,ncol=2) #matrix containing spatio-temporal wind speeds, and diurnal time index
  aa = bb = 1
  for(ii in 1:Period){
    turb.data[aa:(aa+N-1),1] = as.numeric(datas[ii,seq(2,401,2)])
    turb.data[aa:(aa+N-1),2] = rep(bb,N)
    aa = aa + N
    bb = bb + 1
    if(bb > 24){bb = 1}
  }
  colnames(turb.data) = c("Speed","t")
  
  #Fitting harmonic trend
  r = harmonics.fit(turb.data)
  
  #Determining wind direction partition
  dir = as.numeric(mastdata[,2])
  cuts = c(315,45,135,225) #wind direction partitions
  reff = dir[st:end]

  #Finding sample average of wind direction
  circ.dir = circular(reff,type="angles",units="degrees",rotation="clock")
  av.dir = mean.circular(circ.dir,na.rm=T)
  if (as.numeric(av.dir) < 0){av.dir = as.numeric(av.dir) + 360}
  if (av.dir >= cuts[1] & av.dir <= 360){part.id = 1}
  if (av.dir >= 0 & av.dir < cuts[2]){part.id = 1}
  if (av.dir >= cuts[2] & av.dir < cuts[3]){part.id = 2}
  if (av.dir >= cuts[3] & av.dir < cuts[4]){part.id = 3}
  if (av.dir >= cuts[4] & av.dir < cuts[1]){part.id = 4}
  
  #Deciding on the spatial partition
  if (part.id == 1){ #wind is nothernly
    wake = bounds.lat
  }
  if (part.id == 2){ #wind is easterly
    wake = bounds.long
  }
  if (part.id == 3){ #wind is southerly
    wake = bounds.lat
  }
  if (part.id == 4){ #wind is westerly
    wake = bounds.long
  }
  
  #Time lag
  kvalues = pp_kstar[pp]
  
  #Spatial Bandwidth
  bandwidth = pp_band[pp]
  LL = av.dir - bandwidth
  UL = av.dir + bandwidth
  
  #Distance Matrix Computation
  if (part.id == 1 | part.id == 3){dimen = coords$lat}
  if (part.id == 2 | part.id == 4){dimen = coords$long}
  dist_mat = rdist.earth(coords[3:2],miles=F)
  
  #Residual matrix
  Z = matrix(nrow = N,ncol=Period ) #matrix with time periods cols and turbines rows
  a = 1
  for (i in 1:Period){
    Z[,i] = r[a:(a+N-1)] 
    a = a + N
  }
  
  #Asymmetry computation
  asym_list = dist_list = list(length = 2)
  k = kvalues
  vario.mat = matrix(nrow=N,ncol=N)
  for (ii in 1:N){
    out = lapply(seq(1,(Period-k-1),1), function(j) (Z[ii,j+k] - Z[,j])^2)
    vario.mat[ii,]=sapply(seq(1,N,1),function(jj) (1/(2*(Period-k-1)))*sum(sapply(out, `[[`, jj))) #summing the jjth element of each subelement in the list
  }

  for (iterwake in 1:2){
    if (part.id == 1){
      if (iterwake == 1){sel=which(coords$lat>=wake)}
      if (iterwake == 2){sel=which(coords$lat<wake)}}
    if (part.id == 2){
      if (iterwake == 1){sel=which(coords$long>=wake)}
      if (iterwake == 2){sel=which(coords$long<wake)}}
    if (part.id == 3){
      if (iterwake == 1){sel=which(coords$lat<=wake)}
      if (iterwake == 2){sel=which(coords$lat>wake[1])}}
    if (part.id == 4){
      if (iterwake == 1){sel=which(coords$long<=wake)}
      if (iterwake == 2){sel=which(coords$long>wake)}}

    a = 1
    ddiff = vector(length = 1)
    asym = vector(length = 1)
    
    if (part.id == 1){
      
      if (LL < 0){case = 1}
      if (LL >= 0){case = 2}
      
      for (i in sel){
        for (j in sel){
          
          if (dimen[j] < dimen[i]){ #means that i is north of x
            
            if (coords[i,3] < coords[j,3]){ #i is west of j
              diff.lat = (coords[i,2] - coords[j,2])
              diff.long = -(coords[i,3] - coords[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 180 + 90 + angle}
            
            if (coords[i,3] > coords[j,3]){ #i is east of j
              diff.lat =  (coords[i,2] - coords[j,2])
              diff.long = (coords[i,3] - coords[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 - angle}
            
            if (case == 1){
              if (angle > 270 & angle < 360){angle = angle - 360}
            }
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(coords[i,3:2],coords[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    if (part.id == 2){
      for (i in sel){
        for (j in sel){
          if (dimen[j] < dimen[i]){ #means that i is east of x
            
            if (coords[i,2] > coords[j,2]){ #i is north of j
              diff.lat = coords[i,2] - coords[j,2]
              diff.long = coords[i,3] - coords[j,3]
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 - angle}
            
            if (coords[i,2] < coords[j,2]){ #i is south of j
              diff.lat =  -(coords[i,2] - coords[j,2])
              diff.long =  (coords[i,3] - coords[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 + angle}
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(coords[i,3:2],coords[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    if (part.id == 3){
      for (i in sel){
        for (j in sel){
          if (dimen[j] > dimen[i]){ #means that i is south of x
            
            if (coords[i,3] < coords[j,3]){ #i is west of j
              diff.lat = -(coords[i,2] - coords[j,2])
              diff.long = -(coords[i,3] - coords[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 180 + 90 - angle}
            
            if (coords[i,3] > coords[j,3]){ #i is east of j
              diff.lat =  -(coords[i,2] - coords[j,2])
              diff.long =  (coords[i,3] - coords[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 + angle}
            
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(coords[i,3:2],coords[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    if (part.id == 4){
      for (i in sel){
        for (j in sel){
          if (dimen[j] > dimen[i]){ #means that i is west of x
            
            if (coords[i,2] > coords[j,2]){ #i is north of j
              diff.lat = coords[i,2] - coords[j,2]
              diff.long = - coords[i,3] + coords[j,3]
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 270 + angle}
            
            if (coords[i,2] < coords[j,2]){ #i is south of j
              diff.lat =  -(coords[i,2] - coords[j,2])
              diff.long =  - (coords[i,3] - coords[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 270 - angle}
            
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(coords[i,3:2],coords[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    asym_list[[iterwake]] = asym/var(r)
    dist_list[[iterwake]] = ddiff
  } 
  asym_PP[[pp]] = asym_list
  dist_PP[[pp]] = dist_list
  cat(paste("PP is",pp,"out of", length(ids_PP),sep=" "),"\n")
}

#Aggregation of freewind asymmetry
tempp = tempp_d = 0
for (j in 1:length(ids_PP)){
  tempp = c(tempp,asym_PP[[j]][[1]])
  tempp_d = c(tempp_d,dist_PP[[j]][[1]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_free = as.matrix(cbind(tempp_d,tempp))

#Aggregation of waked asymmetry
tempp = tempp_d = 0
for (j in 1:length(ids_PP)){
  tempp = c(tempp,asym_PP[[j]][[2]])
  tempp_d = c(tempp_d,dist_PP[[j]][[2]])
}
tempp = tempp[-1];tempp_d = tempp_d[-1]
agg_wake = as.matrix(cbind(tempp_d,tempp))

bbb = bplot.xy(agg_wake[,1],agg_wake[,2],outline=F,plot=F)
bbb2 = bplot.xy(agg_free[,1],agg_free[,2],outline=F,plot=F)

#***********************************************************************
#********************FINAL PLOTTING*************************************
#***********************************************************************
x = agg_free[,1]
par(cex.main = 1.5,mar=c(5,7,4,4)+0.1,cex.lab=1.2,font.lab=1.2,cex.axis=1.2,las=1)
plot(x, rep(-10,length(x)), type = "p", ylab = " ", xlab = " ", cex = 1, 
     ylim = c(0,0.15), xlim = c(0,20), lwd = 2, pch = 5, axes = F, main = " ",panel.first=grid())
par(las = 0)
axis(1, at = seq(0,20,5),cex.axis=1.5,cex.lab=1.5,lwd=3)
mtext("Distance (Km)", side = 1, line = 3, cex = 1.75, font = 2,lwd=3)
axis(2, pos = , seq(0,0.15,0.05),cex.lab=1.75,cex.axis=1.75,lwd=3)
mtext("Asymmetry in correlation", side = 2, line = 3, cex = 1.75, font = 2,lwd=3)
box(lwd=3,col="black")
lines(bbb$centers[1:10],bbb[[3]]$stats[3,1:10],cex=1.5,pch=19,col="darkred",lwd=2,type="b")
lines(bbb2$centers[1:10],bbb2[[3]]$stats[3,1:10],cex=1.5,type="b",pch=0,lwd=2,col="darkblue")
legend(legend=c("Disturbed wind","Free Wind"),col=c("darkred","darkblue"),pch=c(19,0),cex=1.75,bty="n",lwd=c(2,2),"topleft",y.intersp=0.6)


##Saving important files
saveRDS(asym_PP, "Code Used in the Book/Chapter3/temp/asym_wake.rds")  
saveRDS(dist_PP, "Code Used in the Book/Chapter3/temp/dist_wake.rds") 
