################################################################################
################################################################################
#File Description: 
#This file generates the Period 4 results for Tables 3.5 - 3.8
################################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(circular)
library(Matrix)
library(stats)
library(e1071)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter3/funcs/bin_est.R")
source("Code Used in the Book/Chapter3/funcs/asym_fit.R")
source("Code Used in the Book/Chapter3/funcs/asym_build.R")
source("Code Used in the Book/Chapter3/funcs/SEP_fit.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
bin_width = 0.5 #bin width
vci=3.5 #cut-in speed
vr=13.5 #rated speed

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
N = 200 #number of turbines
Period = 6 #Length of training period in hours
num_test = 4 #Length of testing period in hours
nn = N * (Period + num_test) #number of spatio-temporal data instances
yearlydata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
datas = matrix(nrow = 8760*N,ncol=3) #matrix containing spatio-temporal wind speeds, turbine index, and power
aa  = 1
for(ii in 1:8760){
  datas[aa:(aa+N-1),2] = as.numeric(yearlydata[ii+4,seq(2,401,2)])
  datas[aa:(aa+N-1),1] = seq(1,200,1)
  datas[aa:(aa+N-1),3] = as.numeric(yearlydata[ii+4,seq(3,402,2)])
  aa = aa + N
}
datas = as.data.frame(datas)
colnames(datas) = c("ID","Speed","Power")
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1) #Index of Turbines
coords[,2] = as.numeric(yearlydata[1,2:201]) #Latitude
coords[,3] = as.numeric(yearlydata[2,2:201]) #Longitude
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")
pp4 = read.csv("Code Used in the Book/Chapter3/pp4.csv",header=T)
num_masts = 1
dirs = c(125.5,138.1,140,175.2,120.2,106)
speeds = c(9.99,8.66,10.93,4.24,5.58,8.03)

#***********************************************************************
#*******************PERIOD 4: JUNE 2011*********************************
#***********************************************************************
##Turbine-specific wind speeds and time indices
turb.data = matrix(nrow = nn,ncol=3) #matrix containing spatio-temporal wind speeds, turbine index, and power
aa  = 1
for(ii in 1:(Period+num_test)){
  turb.data[aa:(aa+N-1),1] = as.numeric(pp4[ii,seq(2,401,2)])
  turb.data[aa:(aa+N-1),2] = seq(1,200,1)
  turb.data[aa:(aa+N-1),3] = as.numeric(pp4[ii,seq(3,402,2)])
  aa = aa + N
}

#Training and testing data
train = turb.data[1:(Period*N),]
test = turb.data[(Period*N+1):dim(turb.data)[1],]


#Average wind direction and speeds for training data
av.dir = as.numeric(mean.circular(circular(dirs,type="angles",units="degrees",rotation="clock"),na.rm=T))
if(av.dir < 0){av.dir = av.dir + 360}
st.av_d = av.dir
av.speed = mean(train[,1])

#Determining wind partition based on average wind speed
if (st.av_d >= 315){R_theta = "N"}
if (st.av_d >= 0 & st.av_d < 45){R_theta="N"}
if(st.av_d >= 45 & st.av_d < 135){R_theta="E"}
if(st.av_d >= 135 & st.av_d < 225){R_theta="S"}
if (st.av_d >= 225 & st.av_d < 315){R_theta="W"}

#Building asymmetric covariance
asym.build.res = asym_build(dirs,speeds,R_theta,Period,coords,num_masts)
Cov.mats = asym.build.res[[1]] 
N.vec = asym.build.res[[2]]
coords.mats = asym.build.res[[3]]
Vel = asym.build.res[[4]]
D_list = asym.build.res[[5]]
zz_mat = asym.build.res[[6]]

#model fitting
fit.results = matrix(ncol=10,nrow=num_masts)
loc_mats = list(length = num_masts)
loc_t_mats = list(length = num_masts)
dist_mats = list(length = num_masts)
tdist_mats = list(length = num_masts)
z_vecs = list(length = num_masts)
X_mats = list(length = num_masts)
opt_sets = list(length = num_masts)
final.cov.mats = list(length = num_masts)
mean_est = vector(length = num_masts)
pred_res  = list(length = num_masts)
train_predictions = test_predictions = list(length = num_masts)
colnames(train) = colnames(test) = c("Speed","ID","Power")
train_PP = as.data.frame(train)
test_PP = as.data.frame(test)

for (modd in 1:num_masts){
  z = train_PP$Speed*3.6 
  X.m = matrix(nrow=length(z),ncol=1)
  X.m[,1] = rep(0,length(z)) 
  z = z - mean(z) 
  coords_modd = coords.mats[[modd]]
  loc_modd = matrix( rep( t( coords_modd[,3:2] ) , Period ) , ncol = ncol(coords_modd[,2:3]) , byrow = TRUE )
  loc_t_modd = matrix(nrow=dim(loc_modd)[1],ncol=2)
  x1 = 0
  for (i in 1:Period){
    x2 = rep(i-1,N.vec[modd])
    x1 = c(x1,x2)
  }
  x1 = x1[-1]
  loc_t_modd[,1] = x1
  loc_t_modd[,2] = rep(0,dim(loc_modd)[1])
  spatdist_modd = (rdist.earth(loc_modd,miles=F))#spatial distance matrix
  tempdist_modd = rdist(loc_t_modd) #temporal distance matrix
  starting.seed = c(log(var(z)),rep(-1,5),2,-3) #starting point for MLE
  
  asym_signal = 1
  cat("MODEL FITTING - MAY TAKE FEW MINUTES","\n")
  asym.fit.res = asym_fit(z,tempdist_modd,spatdist_modd,1,10^-5,300,starting.seed,0,asym_signal)
  
  #Recovering optimal parameters and other important values
  phi = asym.fit.res[[1]][1]
  nu = asym.fit.res[[1]][2]
  c = asym.fit.res[[1]][3]
  a = asym.fit.res[[1]][4]
  alpha = asym.fit.res[[1]][5]
  delta = asym.fit.res[[1]][6]
  beta = asym.fit.res[[1]][7]
  lambda = asym.fit.res[[1]][8]
  LLs = as.numeric(as.vector(asym.fit.res[[5]]))
  fit.results[modd,] = c(phi,nu,alpha,a,c,delta,beta,lambda,LLs[1],LLs[2])
  loc_mats[[modd]] = loc_modd
  loc_t_mats[[modd]] = loc_t_modd
  dist_mats[[modd]] = spatdist_modd
  tdist_mats[[modd]] = tempdist_modd
  z_vecs[[modd]] = z
  X_mats[[modd]] = X.m
  final.cov.mats[[modd]] = asym.fit.res[[2]]
  mean_est[modd] = asym.fit.res[[4]][1,1]
  S = asym.fit.res[[2]]
  
  #prediction
  trainvals = vector(length = Period*N.vec[modd])
  K=S
  M = X.m
  Kinv = solve(K)
  testvals = err = vector(length = num_test*N.vec[modd])
  ones = diag(1,nrow=2,ncol=2)
  idd = seq(1,200,1)
  idd_first = idd[which(idd <= 200)]
  aaaa = 1
  idd_all = vector(length = N.vec[modd]*num_test)
  for (kkk in 1:num_test){
    idd_all[aaaa:(aaaa+length(idd_first)-1)] = idd_first + ((kkk-1)*200)
    aaaa = aaaa + length(idd_first)
  }
  test_modd = test_PP[idd_all,]
  loc_test_modd = do.call(rbind, replicate(num_test, loc_modd[1:N.vec[modd],], simplify=FALSE))
  loc_t_test = matrix(nrow=dim(loc_test_modd)[1],ncol=2)
  ccc = 1
  for (i in 1:num_test){
    loc_t_test[ccc:(ccc+N.vec[modd]-1),1] =  rep(tail(loc_t_modd,1)[1] + i,N.vec[modd])
    ccc = ccc + N.vec[modd]
  }
  loc_t_test[,2] = rep(0,num_test*N.vec[modd])
  
  cat("OBTAINING PREDICTIONS","\n")
  for (i in 1:length(testvals)){
    m = 0
    test.coords = loc_test_modd[i,]
    test.t = loc_t_test[i,]
    t1 = matrix(nrow=1,ncol=2)
    t1[1,] = test.coords
    sdist_vec = as.vector(rdist.earth(t1,loc_modd,miles=F))
    t1 = matrix(nrow=1,ncol=2)
    t1[1,] = test.t
    tdist_vec = as.vector(rdist(t1,loc_t_modd))
    k1 = (1-nu)/(1 + a* (tdist_vec)^(2*alpha))
    k2 = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(beta/2)))
    k2[which(sdist_vec == 0)] = k2[which(sdist_vec == 0)] + (nu/(1-nu))
    k12 = k1 * k2
    r = vector(length = length(loc_modd[,1]))
    for (j in 1:length(loc_modd[,1])){
      
      V = Vel[modd,]
      D = D_list[[modd]]
      zz = zz_mat[modd,]
      
      h2 = loc_test_modd[i,2] - loc_modd[j,2] #diff in lat
      h1 = loc_test_modd[i,1] - loc_modd[j,1] #diff in lon
      hh = c(h1,h2)
      tt = loc_t_test[i,1] - loc_t_modd[j,1]
      trans= matrix(hh + tt*V*zz,nrow=1,ncol=2)
      r[j] = (1/sqrt(det(ones + ((tt)^2)*D)))*exp(-((as.numeric(trans %*% solve(ones + ((tt)^2)*D) %*% t(trans)))))
    }
    
    
    k = phi * ( ((1-lambda)*k12) + (lambda*r))
    k = as.matrix(k,nrow=dim(term1a)[2],ncol=1)
    
    lam = Kinv %*% k  
    obs = as.matrix(z)
    pred = (t(lam) %*% obs) 

    pred = pred + mean(train_PP$Speed*3.6)

    testvals[i] = pred/3.6
    err[i] = (testvals[i] - test_modd$Speed[i])^2
   
  }
  test_predictions[[modd]] = testvals
  preds = matrix(nrow=length(testvals),ncol=2)
  preds[,1] = testvals
  preds[,2] = test_modd$Speed
  
  pred_res[[modd]] = preds
}

#Persistence PER
pred_pers = tail(train_PP$Speed,200);pred_pers = rep(pred_pers,times=num_test)
aa = 1;err_pers=vector(length=(num_test*200))
for (i in 1:num_test){
  err_pers[aa:(aa+199)] = (tail(train_PP$Speed,200) - test_PP$Speed[aa:(aa+199)])^2
  aa = aa + 200
}

#ASYM
fin.res = matrix(nrow=(num_test*200),ncol=2)
fin.res[,1:2] = pred_res[[1]]
pred_asym = fin.res[,1]
err_asym =  (fin.res[,1] - fin.res[,2])^2

#Prediction using ARMA, SVM MODELS
preds_arma = preds_svm  = matrix(ncol=num_test,nrow=N)
indices = seq(1,N,1)
for (i in 1:N){
  training = train[which(train[,2] == indices[i]),1]
  ##ARMA(1,1)
  armafit = arima(training,order=c(1,0,1),method="ML") 
  datt2 = matrix(c(training[2:6],training[1:5]),nrow=5,ncol=2)
  colnames(datt2) = c("Y","lag1")
  testdatt2 = matrix(c(0,datt2[5,1]),nrow=1,ncol=2)
  colnames(testdatt2) = c("Y","lag1")
  preds_arma[i,] = as.numeric(predict(armafit,num_test)$pred)
  ##SVM
  svmfit = svm(Y ~ lag1, data=datt2,kernel="radial")
  for(ii in 1:4){
    predictedY = predict(svmfit,testdatt2)
    preds_svm[i,ii] = tail(as.numeric(predictedY),1)
    testdatt2 = matrix(c(rep(0,ii+1),testdatt2[,2],tail(as.numeric(predictedY),1)),nrow=ii+1,ncol=2)
    colnames(testdatt2) = c("Y","lag1")
  }
}
pred_arma = c(preds_arma[,1:num_test]);err_arma = (test[,1] - pred_arma)^2
pred_svm = c(preds_svm[,1:num_test]);err_svm = (test[,1] - pred_svm)^2

###Prediction using HYB MODEL
#Obtaining trainvals
cat("OBTAINING PREDICTIONS AT TRAINING POINTS","\n")
for (i in 1:length(trainvals)){
  m = 0
  test.coords = loc_modd[i,]
  test.t = loc_t_modd[i,]
  t1 = matrix(nrow=1,ncol=2)
  t1[1,] = test.coords
  sdist_vec = as.vector(rdist.earth(t1,loc_modd,miles=F))
  t1 = matrix(nrow=1,ncol=2)
  t1[1,] = test.t
  tdist_vec = as.vector(rdist(t1,loc_t_modd))
  k1 = (1-nu)/(1 + a* (tdist_vec)^(2*alpha))
  k2 = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(beta/2)))
  k2[which(sdist_vec == 0)] = k2[which(sdist_vec == 0)] + (nu/(1-nu))
  k12 = k1 * k2
  r = vector(length = length(loc_modd[,1]))
  for (j in 1:length(loc_modd[,1])){
    
    V = Vel[modd,]
    D = D_list[[modd]]
    zz = zz_mat[modd,]
    
    h2 = loc_modd[i,2] - loc_modd[j,2] #diff in lat
    h1 = loc_modd[i,1] - loc_modd[j,1] #diff in lon
    hh = c(h1,h2)
    tt = loc_t_modd[i,1] - loc_t_modd[j,1]
    trans= matrix(hh + tt*V*zz,nrow=1,ncol=2)
    r[j] = (1/sqrt(det(ones + ((tt)^2)*D)))*exp(-((as.numeric(trans %*% solve(ones + ((tt)^2)*D) %*% t(trans)))))
  }
  
  
  k = phi * ( ((1-lambda)*k12) + (lambda*r))
  k = as.matrix(k,nrow=dim(term1a)[2],ncol=1)
  
  lam = Kinv %*% k  

  obs = as.matrix(z)
  pred = (t(lam) %*% obs) 

  pred = pred + mean(train_PP$Speed*3.6)

  trainvals[i] = pred/3.6
}
#Fitting SVM to residuals of ASYM
res.asym = trainvals - train[,1]
datt = matrix(nrow=length(res.asym),ncol=4)
datt[,1] = res.asym;datt[,2] = rep(coords[,2],times=6)
datt[,3] = rep(coords[,3],times=6);datt[,4] = rep(seq(1,6,1),each=200)
colnames(datt) = c("Y","lat","long","Time")
testdatt = matrix(nrow=200*4,ncol=4)
testdatt[,1] = rep(0,800);testdatt[,2] = rep(coords[,2],times=4)
testdatt[,3] = rep(coords[,3],times=4);testdatt[,4] = rep(seq(7,10,1),each=200)
colnames(testdatt) = c("Y","lat","long","Time")
svmfit2 = svm(Y~lat+long+Time,datt,kernel='radial',gamma=.01)
predictedY = predict(svmfit2,testdatt)
pred_hybrid2 = as.numeric(pred_asym+predictedY);err_hyb = (test[,1] - pred_hybrid2)^2
err_hyb =  (pred_hybrid2 - fin.res[,2])^2


##SEP fit
cat("FITTING THE SEPARABLE MODEL (SEP) - MAY TAKE FEW MINUTES","\n")
sep.fit.res = SEP_fit(z,tempdist_modd,spatdist_modd,1,10^-5,300)
phi = sep.fit.res[[1]][1]
nu = sep.fit.res[[1]][2]
c = sep.fit.res[[1]][3]
a = sep.fit.res[[1]][4]
alpha = sep.fit.res[[1]][5]
delta = sep.fit.res[[1]][6]
beta = sep.fit.res[[1]][7]
lambda = sep.fit.res[[1]][8]
LLsep = as.numeric(as.vector(sep.fit.res[[5]]))
fit.results[modd,] = c(phi,nu,alpha,a,c,delta,beta,lambda,LLs[1],LLs[2])
loc_mats[[modd]] = loc_modd
loc_t_mats[[modd]] = loc_t_modd
dist_mats[[modd]] = spatdist_modd
tdist_mats[[modd]] = tempdist_modd
z_vecs[[modd]] = z
X_mats[[modd]] = X.m
final.cov.mats[[modd]] = sep.fit.res[[2]]
mean_est[modd] = sep.fit.res[[4]][1,1]
S = sep.fit.res[[2]]

#Reporting all LL values
LL_mat = matrix(nrow=1,ncol=2)
colnames(LL_mat) = c("ASYM","SEP")
LL_mat[1,] = c(LLs[1],LLsep[1])

#prediction
trainvals = vector(length = Period*N.vec[modd])
K=S
M = X.m
Kinv = solve(K)
testvals = err =  vector(length = num_test*N.vec[modd])
ones = diag(1,nrow=2,ncol=2)
idd = seq(1,200,1)
idd_first = idd[which(idd <= 200)]
aaaa = 1
idd_all = vector(length = N.vec[modd]*num_test)
for (kkk in 1:num_test){
  idd_all[aaaa:(aaaa+length(idd_first)-1)] = idd_first + ((kkk-1)*200)
  aaaa = aaaa + length(idd_first)
}
test_modd = test_PP[idd_all,]
loc_test_modd = do.call(rbind, replicate(num_test, loc_modd[1:N.vec[modd],], simplify=FALSE))
loc_t_test = matrix(nrow=dim(loc_test_modd)[1],ncol=2)
ccc = 1
for (i in 1:num_test){
  loc_t_test[ccc:(ccc+N.vec[modd]-1),1] =  rep(tail(loc_t_modd,1)[1] + i,N.vec[modd])
  ccc = ccc + N.vec[modd]
}
loc_t_test[,2] = rep(0,num_test*N.vec[modd])

cat("OBTAINING PREDICTIONS FOR THE SEPARABLE MODEL","\n")
for (i in 1:length(testvals)){
  m = 0
  test.coords = loc_test_modd[i,]
  test.t = loc_t_test[i,]
  t1 = matrix(nrow=1,ncol=2)
  t1[1,] = test.coords
  sdist_vec = as.vector(rdist.earth(t1,loc_modd,miles=F))
  t1 = matrix(nrow=1,ncol=2)
  t1[1,] = test.t
  tdist_vec = as.vector(rdist(t1,loc_t_modd))
  k1 = (1-nu)/(1 + a* (tdist_vec)^(2*alpha))
  k2 = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(beta/2)))
  k2[which(sdist_vec == 0)] = k2[which(sdist_vec == 0)] + (nu/(1-nu))
  k12 = k1 * k2
  r = vector(length = length(loc_modd[,1]))
  for (j in 1:length(loc_modd[,1])){
    
    V = Vel[modd,]
    D = D_list[[modd]]
    zz = zz_mat[modd,]
    
    h2 = loc_test_modd[i,2] - loc_modd[j,2] #diff in lat
    h1 = loc_test_modd[i,1] - loc_modd[j,1] #diff in lon
    hh = c(h1,h2)
    tt = loc_t_test[i,1] - loc_t_modd[j,1]
    trans= matrix(hh + tt*V*zz,nrow=1,ncol=2)
    r[j] = (1/sqrt(det(ones + ((tt)^2)*D)))*exp(-((as.numeric(trans %*% solve(ones + ((tt)^2)*D) %*% t(trans)))))
  }
  
  
  k = phi * ( ((1-lambda)*k12) + (lambda*r))
  k = as.matrix(k,nrow=dim(term1a)[2],ncol=1)
  
  
  lam = Kinv %*% k  

  obs = as.matrix(z)
  pred = (t(lam) %*% obs) 

  pred = pred + mean(train_PP$Speed*3.6)

  testvals[i] = pred/3.6
  err[i] = (testvals[i] - test_modd$Speed[i])^2
}
test_predictions[[modd]] = testvals
preds = matrix(nrow=length(testvals),ncol=2)
preds[,1] = testvals
preds[,2] = test_modd$Speed
pred_res[[modd]] = preds
fin.res.sep = matrix(nrow=(num_test*200),ncol=2)
fin.res.sep[,1:2] = pred_res[[1]]
pred_sep = fin.res.sep[,1]
err_sep =  (fin.res.sep[,1] - fin.res.sep[,2])^2


##Reporting RMSE and MAE for wind speed forecasting in Period 4
RMSE = MAE = matrix(nrow=6,ncol=5)
colnames(RMSE) = colnames(MAE) = c("h=1","h=2","h=3","h=4","Average")
rownames(RMSE) = rownames(MAE) = c("ASYM","SEP","PER","ARMA(1,1)","SVM","HYB")
#Prediction error by hour
aa = 1
for(i in 1:num_test){
  RMSE[1,i] = sqrt(mean(err_asym[aa:(aa+N-1)]))
  RMSE[2,i] = sqrt(mean(err_sep[aa:(aa+N-1)]))
  RMSE[3,i] = sqrt(mean(err_pers[aa:(aa+N-1)]))
  RMSE[4,i] = sqrt(mean(err_arma[aa:(aa+N-1)]))
  RMSE[5,i] = sqrt(mean(err_svm[aa:(aa+N-1)]))
  RMSE[6,i] = sqrt(mean(err_hyb[aa:(aa+N-1)]))
  MAE[1,i] = mean(sqrt(err_asym[aa:(aa+N-1)]))
  MAE[2,i] = mean(sqrt(err_sep[aa:(aa+N-1)]))
  MAE[3,i] = mean(sqrt(err_pers[aa:(aa+N-1)]))
  MAE[4,i] = mean(sqrt(err_arma[aa:(aa+N-1)]))
  MAE[5,i] = mean(sqrt(err_svm[aa:(aa+N-1)]))
  MAE[6,i] = mean(sqrt(err_hyb[aa:(aa+N-1)]))
  
  aa = aa + N
}

RMSE[1,5] = sqrt(mean(err_asym))
RMSE[2,5] = sqrt(mean(err_sep))
RMSE[3,5] = sqrt(mean(err_pers))
RMSE[4,5] = sqrt(mean(err_arma))
RMSE[5,5] = sqrt(mean(err_svm))
RMSE[6,5] = sqrt(mean(err_hyb))
MAE[1,5] = mean(sqrt(err_asym))
MAE[2,5] = mean(sqrt(err_sep))
MAE[3,5] = mean(sqrt(err_pers))
MAE[4,5] = mean(sqrt(err_arma))
MAE[5,5] = mean(sqrt(err_svm))
MAE[6,5] = mean(sqrt(err_hyb))


##Power domain conversion using binning method
powerlist = bin_est(datas,bw=bin_width,cutin = vci,rated = vr,numturbs = N)
vbin = as.numeric(powerlist[[1]])
P.est = powerlist[[2]]
pred_sep = fin.res.sep[,1]

##Power estimation
indices = seq(1,N,1)
indices = rep(indices,times=num_test)
P.est_test = P_true_test = P_binest_test = P_svm = P_sep  = P_pers = P_arma = P_hyb = vector(length = (num_test*N))
for (i in 1:(num_test*200)){
  index = indices[i]
  P.est_test[i] = P.est[[index]][which(sort(c(pred_asym[i],vbin)) == pred_asym[i])]
  P_true_test[i] = test[i,3]
  P_sep[i] = P.est[[index]][which(sort(c(pred_sep[i],vbin)) == pred_sep[i])]
  P_pers[i] = P.est[[index]][which(sort(c(pred_pers[i],vbin)) == pred_pers[i])[1]]
  P_arma[i] = P.est[[index]][which(sort(c(pred_arma[i],vbin)) == pred_arma[i])[1]]
  P_hyb[i] = P.est[[index]][which(sort(c(pred_hybrid2[i],vbin)) == pred_hybrid2[i])[1]]
  P_svm[i] = P.est[[index]][which(sort(c(pred_svm[i],vbin)) == pred_svm[i])[1]]
}
outlier.id = unique(c(which(is.na(P_true_test) == T),which(P_true_test <= 0)))
if(length(outlier.id) != 0){P_true_test[outlier.id] = NA}
RMSE_p = RMSE
RMSE_p[1,5] = sqrt(mean((P.est_test - P_true_test)^2,na.rm=T))
RMSE_p[2,5] = sqrt(mean((P_sep - P_true_test)^2,na.rm=T))
RMSE_p[3,5] = sqrt(mean((P_pers - P_true_test)^2,na.rm=T))
RMSE_p[4,5] = sqrt(mean((P_arma - P_true_test)^2,na.rm=T))
RMSE_p[5,5] = sqrt(mean((P_svm - P_true_test)^2,na.rm=T))
RMSE_p[6,5] = sqrt(mean((P_hyb - P_true_test)^2,na.rm=T))


##Computing forecasting error per hour
err.p = matrix(nrow=num_test,ncol=6)
aa=1;bb=200
for (i in 1:num_test){
  ind.pout = outlier.id[which(outlier.id>=aa & outlier.id<=bb)]-aa+1
  err.p[i,1] = sqrt(mean((P.est_test[aa:bb][-ind.pout] - P_true_test[aa:bb][-ind.pout])^2,na.rm=T))
  err.p[i,2] = sqrt(mean((P_sep[aa:bb][-ind.pout] - P_true_test[aa:bb][-ind.pout])^2,na.rm=T))
  err.p[i,3] = sqrt(mean((P_pers[aa:bb][-ind.pout] - P_true_test[aa:bb][-ind.pout])^2,na.rm=T))
  err.p[i,4] = sqrt(mean((P_arma[aa:bb][-ind.pout] - P_true_test[aa:bb][-ind.pout])^2,na.rm=T))
  err.p[i,5] = sqrt(mean((P_svm[aa:bb][-ind.pout] - P_true_test[aa:bb][-ind.pout])^2,na.rm=T))
  err.p[i,6] = sqrt(mean((P_hyb[aa:bb][-ind.pout] - P_true_test[aa:bb][-ind.pout])^2,na.rm=T))
  
  aa = aa + 200
  bb = bb + 200
}
RMSE_p[1:6,1:4] = t(err.p)


##Saving important files
write.table(LL_mat[1,],file="Code Used in the Book/Chapter3/temp/Table3.5_Period4.txt",col.names = F,row.names = F)
write.table(RMSE,file="Code Used in the Book/Chapter3/temp/Table3.6_Period4.txt",col.names = F,row.names = F)
write.table(MAE,file="Code Used in the Book/Chapter3/temp/Table3.7_Period4.txt",col.names = F,row.names = F)
write.table(RMSE_p,file="Code Used in the Book/Chapter3/temp/Table3.8_Period4.txt",col.names = F,row.names = F)
