
# ////////

rm(list=ls())
library('getopt')
GSTAR = Sys.getenv("GSTAR")
code_dir = paste(GSTAR,"codes/", sep="")
data_dir =  paste(GSTAR,"data/", sep="")




spec = matrix(c(
	'help' , 'h', 0, "logical",
	'config' , 'c', 1, "character",
	'year', 'y', 1, "integer",
	'save', 's', 2, "integer",
	'crps', 'r', 2, "integer",
	'varalpha', 'v', 2, "integer"
	), byrow=TRUE, ncol=4)
opt = getopt(spec)


if ( !is.null(opt$help) ) {
cat(getopt(spec, usage=TRUE))
	print("use -c for config 'm'/'s'/'l'")
	print("use -y for year 1/2")
	print("use -s for saving 0/1 (optional)")
	print("use -r for crps 0/1 (optional)")
	print("use -v for varying alpha 0/1 (optional)")
	q(status=1)
}

if ( is.null(opt$config ) ) { 
	print("using mini config")
	opt$config = 'm' 

}
if ( is.null(opt$year ) ) { 
	print("using data from year 2009")
	opt$year = 1 
}
if ( is.null(opt$save ) ) { opt$save = 1 }

if ( is.null(opt$crps ) ) { opt$crps = 0 }

if ( is.null(opt$varalpha ) ) { opt$varalpha = 0 }


# load the functions
source(paste0(code_dir,"functions.R"))

if (opt$year ==1 ){
	year <- 2009
} else if (opt$year ==2) {
	year <- 2010
}

if (opt$crps==1) {
	crps <- data.table()
}

print(paste0("Reading data for year: ", year))
dt <- fread(paste0(data_dir,sprintf("Wind Spatio-Temporal Dataset1(%d, Ave).csv", year)))
std <- fread(paste0(data_dir,sprintf("Wind Spatio-Temporal Dataset1(%d, Stdev).csv", year)))

config <- fread(paste0(data_dir,sprintf("config_%d.csv", year)))

if (opt$config == "m") {
	print("using mini conifg: only 5 rows.")
	config = config[seq(1, nrow(config), nrow(config)/5)]
} else if (opt$config == "s") {
	print("using small conifg: only 15 rows.")
	config = config[seq(1, nrow(config), nrow(config)/15)]
} else if (opt$config == "l") {
	print(sprintf("using LARGE conifg. %d rows", nrow(config)))
}

config <- as.matrix(config)
print(config)

beta = 0.85
TRUNCATE = TRUE
gamma = 0.000001
alpha <- 0.73

month = 30*24
test_period = 24
T = month - test_period



TG_coor=get_TG_coor(filename="Wind Spatio-Temporal Dataset1(Coordinates).csv")

I = 120
TG_coor <- as.matrix(TG_coor[1:I,])
pairwise_dist <- pdist(TG_coor)
dt <- dt[, 1:I]
std <- std[, 1:I]




all_errors = data.table()

for (i_counter in 1:nrow(config)) {	
	param <- config[i_counter, ]
	im <- as.numeric(param[1])
	h <- as.numeric(param[2])
	turbine <- as.numeric(param[3])
	epoch <- as.numeric(param[4])
	run_arma <- as.numeric(param[5])



	print("================")
	print("================")
	print("================")
	print("================")

	
	

	print(sprintf("starting for h=%i, e=%i, turbine=%i, im=%i",h, epoch, turbine, im))
	dt_month_epoch = dt[((im-1)*month+1):(im*month),]
	dt_month_epoch[,epoch:=(seq(0,nrow(dt_month_epoch)-1) %% 24)%/%6+1]

	
	std_month_epoch = std[((im-1)*month+1):(im*month),]
	std_month_epoch[,epoch:=(seq(0,nrow(std_month_epoch)-1) %% 24)%/%6+1]


	#################
	#################
	start_time <- Sys.time()

	rho<-calc_rho(dt_month_epoch[1:T,-ncol(dt_month_epoch),with=FALSE], beta = beta)

	J = find_neighborhood(rho, turbine)

	while(nrow(J) >  15 & TRUNCATE){
		beta = 1.01 * beta
		print(paste0("truncating the neighborhood size...increasing beta to ", beta))
		print(beta)
		J = find_neighborhood(
			calc_rho(
				dt_month_epoch[1:T,-ncol(dt_month_epoch),with=FALSE], beta = beta),turbine)
	}

	while(nrow(J) < 2){
		beta = 0.99 * beta
		print(paste0("no neighbor found...lowering beta to ", beta))
		print(beta)
		J = find_neighborhood(
			calc_rho(
				dt_month_epoch[1:T,-ncol(dt_month_epoch),with=FALSE], beta = beta),turbine)
	}
	beta = 0.85
	o <- f_do_optimization(dt_month_epoch[,c(paste("V",J$row,sep=""), "epoch"),with=FALSE], 
		T, cbind(TG_coor[J$row,], J$row),
		 rho[J$row,J$row], turbine=turbine, epoch=epoch, h = h, gamma = gamma)

	std_pred <- f_estimate_std(std_month_epoch, dt_month_epoch,  T = T, turbine=turbine, epoch=epoch)

	output <- f_calc_error(dt_month_epoch[,c(paste("V",J$row,sep=""), "epoch"),with=FALSE], 
	 	o, std_pred, T, cbind(TG_coor[J$row,], J$row), rho[J$row,J$row], 
	 	turbine=turbine, epoch=epoch, h = h, test_period = test_period, alpha = alpha, cutout = 20)
	

	end_time <- Sys.time()
	print("gstar time:")
	print(end_time - start_time)
	
	#################
	#################
	#arma
	if (run_arma == 1) {	
		start_time <- Sys.time()
	
		output_arma <- f_calc_arma_error(dt_month_epoch[,c(paste("V",turbine,sep=""), "epoch"),with=FALSE], T = T, 
			turbine = turbine, epoch = epoch, h = h, 
			test_period = test_period, alpha = alpha, crps = opt$crps)

		end_time <- Sys.time()
		if (opt$crps == 1 & turbine %in% seq(1,120,3)) {
			# seq(1,120,3) are forty randomly selected turbines
			print("calc crps...")
			tmp_crps <- output$dt_pred
			
			std_pred_epcoh = as.data.table(std_pred)
			names(std_pred_epcoh)[1] <- 'std'
			std_pred_epcoh[, e := rep(1:4, each=6)]

			crps_epoch = max(tmp_crps$epoch)
			std_pred_epcoh <- std_pred_epcoh[e == crps_epoch]
			
			tmp_crps[, std := std_pred_epcoh$std]
			tmp_crps[, pred_arma := output_arma$dt_pred$pred_arma_seas]
			tmp_crps[, std_arma := output_arma$dt_pred$std_arma_seas]
			tmp_crps[, h := h]
			tmp_crps[, i := i_counter]
			crps <- rbindlist(list(crps, tmp_crps), use.names = TRUE)
			}

		print("arma time:")
		print(end_time - start_time)
	}
	
	#################
	#################
	#var

	start_time <- Sys.time()
	out_var <- f_calc_var_error(dt_month_epoch, T = T, pairwise_dist = pairwise_dist,
		turbine = turbine, epoch = epoch, h = h, test_period = test_period, alpha = alpha)

	end_time <- Sys.time()
	print("var time:")
	print(end_time - start_time)
	#################
	#################
	if (run_arma) {	
		tmp = cbind(output$dt_error,
			output_arma$dt_error[,.(yarma,yarmaseas,arma_error,arma_seas_error)],
			out_var$dt_error[,.(yvar,var_error)])		
	} else {
		tmp = cbind(output$dt_error,
			out_var$dt_error[,.(yvar,var_error)])
	}		
	tmp[,h:=h]
	tmp[,e:=epoch]
	tmp[,turbine:=turbine]
	tmp[,im:=im]
	all_errors = rbindlist(list(all_errors, tmp), use.names = TRUE, fill = TRUE)


	print(sprintf("done for h=%i ; epoch=%i ; turbine=%i ; im=%i ", h, epoch, turbine, im))	

	print("================")
	print("================")
	print("================")
	print("================")
	print("++++++++++++++++")
}

print("mean prediction error... ")

print(all_errors[, lapply(.SD, mean, na.rm = TRUE), by = h][, .(h, gstar_error, 
	pers_error,  arma_error, 
	arma_seas_error, var_error)][order(h)])

print("standard deviation of prediction error... ")


if (opt$config =="m" | opt$config =="s"){
	print(all_errors[, lapply(.SD, sd, na.rm = TRUE), by = h][, .(h, gstar_error, 
		pers_error,  arma_error, 
		arma_seas_error, var_error)][order(h)])
} else {
	print(all_errors[, lapply(.SD, mean, na.rm = T), by = .(im, turbine, e, h)][, 
		lapply(.SD, sd, na.rm = T), by = h][, .(h, gstar_error, 
		pers_error,  arma_error, 
		arma_seas_error, var_error)][order(h)])

}

if (opt$crps == 1) {
	if (nrow(crps) > 0) {
		crps[,crps_gstar:=mapply(f_crps, stest=y, smean=pred, sstd=std, alpha = 0.5)]
		crps[,crps_arma:=mapply(f_crps, stest=y, smean=pred_arma, sstd=std_arma, alpha =0.5)]
		print("crps")
		print(crps[, lapply(.SD, mean), by = h][,.(h, crps_gstar,crps_arma)])
		# crps[, lapply(.SD, mean), by =.(h,epoch)][, lapply(.SD, sd), by = h][,.(crps_gstar,crps_arma)]
	}
}

if(opt$varalpha == 1) {
	print("calculating for varying alpha")
	all_errors[, gstar_error_alpha_all := 0]
	all_errors[, pers_error_alpha_all := 0]
	all_alpha = seq(0.61, 0.81, 0.02)
	for (alpha in all_alpha){
		all_errors[,gstar_error_alpha :=apply(all_errors, 1, function(x) pc_error(x[1], x[2], alpha = alpha))]
		all_errors[,pers_error_alpha :=apply(all_errors, 1, function(x) pc_error(x[1], x[3], alpha = alpha))]	

		all_errors[, gstar_error_alpha_all := gstar_error_alpha + gstar_error_alpha_all]
		all_errors[, pers_error_alpha_all := pers_error_alpha + pers_error_alpha_all]
	}

	all_errors[, gstar_error_alpha_all := gstar_error_alpha_all / length(all_alpha)]
	all_errors[, pers_error_alpha_all := pers_error_alpha_all / length(all_alpha)]

	all_errors[, gstar_error_alpha := NULL][, pers_error_alpha := NULL]

	print("mean:")
	print(all_errors[turbine %in% seq(1,100), lapply(.SD, mean, na.rm = TRUE), by = h][, .(h, gstar_error_alpha_all, 
	pers_error_alpha_all)][order(h)])

	if (opt$config =="m" | opt$config =="s"){
		print("std:")
		print(all_errors[turbine %in% seq(1,100), lapply(.SD, sd, na.rm = TRUE), by = h][, .(h, gstar_error_alpha_all, 
			pers_error_alpha_all)][order(h)])
	} else {
		print("std:")
		print(all_errors[turbine %in% seq(1,100), lapply(.SD, mean, na.rm = T), by = .(im, turbine, e, h)][, 
			lapply(.SD, sd, na.rm = T), by = h][, .(h, gstar_error_alpha_all, 
			pers_error_alpha_all)][order(h)])
	}

}


if (opt$save == 1) {
	out_dir =  paste(GSTAR,"output/", sep="")
	system(sprintf("mkdir -p %s ", out_dir))
	print("saving the results")
	write.table(all_errors, 
		paste0(out_dir,sprintf("results_config_%s_year_%i.csv", opt$config, year)), row.names = FALSE)

}
