
rm(list=ls())
library('getopt')
GSTAR = Sys.getenv("GSTAR")
code_dir = paste(GSTAR,"codes/", sep="")
data_dir =  paste(GSTAR,"data/", sep="")


spec = matrix(c(
	'help' , 'h', 0, "logical",
	'turbine' , 't', 1, "integer"
	), byrow=TRUE, ncol=4)
opt = getopt(spec)

if ( !is.null(opt$help) ) {
	cat(getopt(spec, usage=TRUE))
	print("use -t for turbine: suggstions 15; 23; 55")
	q(status=1)
}

if(is.null(opt$turbine )){
	print("turbine number must be provided: Rscrip plots_.R -t [NUMBER]")
	stop()
}

source(paste0(code_dir,"functions.R"))

year <- 2009
dt <- fread(paste0(data_dir,sprintf("Wind Spatio-Temporal Dataset1(%d, Ave).csv", year)))

im <- 1
month = 30*24
dt_month <- dt[((im-1)*month+1):(im*month),]


beta = 0.85


TG_coor=get_TG_coor(filename="Wind Spatio-Temporal Dataset1(Coordinates).csv")

I = 120
TG_coor <- as.matrix(TG_coor[1:I,])
dt_TG_coor = data.table(TG_coor)


turbine = opt$turbine # 15 (7), 16, 17, 18, 23 (7), 40, 44, 45, 46, 55 (7), 63, 78, 88, 90, 110, 111
print(turbine)
im <- 1


rho<-calc_rho(dt_month, beta = beta)

J = find_neighborhood(rho, turbine)


gg <- ggplot(dt_TG_coor, aes(x=X1, y=X2)) + 
		geom_point(shape=21, fill="grey", color="grey", size=3) +
		geom_point(data = dt_TG_coor[J$row], shape=22, fill="blue", color="red", size=4) +
		geom_point(data = dt_TG_coor[turbine], shape=21, fill="blue", color="red", size=5) +
		labs(subtitle="", 
		       y="Latitude", 
		       x="Longitude"
		       # title="Informative Neighborhood"
		       ) +
		theme(legend.position="top",
			  axis.text.x=element_blank(),
		      axis.text.y=element_blank(),
		      axis.ticks=element_blank(),
		      axis.title.x = element_text(size = 16, face = "bold"),
		      axis.title.y = element_text(size = 16, face = "bold"),
		      plot.title = element_text(size = 20, face = "bold", color = "darkgreen")) 
	
# plot(gg)

plot_dir =  paste(GSTAR,"plots/", sep="")
system(sprintf("mkdir -p %s ", plot_dir))
print(sprintf("saving the results at %s", plot_dir))
ggsave(sprintf("%splot_t%i.jpeg", plot_dir,opt$turbine), 
	device = 'jpeg', width = 20, height = 20, units = "cm")

