normalize = function(ws, m = 0.5){
  library(stringr)
  str = str_split_fixed(ws$Time," ",2)
  wsdata = data.frame(str,ws[,2])
  colnames(wsdata) <- c("Date","Time","WindSpeed")
  days = length(unique(wsdata[,1]))
  hour = unique(wsdata[,2])
  V_prime = (wsdata[,3])^m
  wsdata = data.frame(wsdata,V_prime)
  V_prime_bar = NULL
  for (i in 1:length(hour)){
    V_prime_bar[i] = mean(wsdata$V_prime[which(wsdata$Time==hour[i])])
  }
  s = NULL
  for (i in 1:length(hour)){
    s[i] = sd(wsdata$V_prime[which(wsdata$Time==hour[i])])
  }
  NormalizedWindspeed = NULL
  for (i in 1:nrow(wsdata)){
    NormalizedWindspeed[i] = ((wsdata$V_prime[i]) -V_prime_bar[which(hour == wsdata$Time[i])])/s[which(hour == wsdata$Time[i])]
  }
  wsdata  = data.frame(wsdata,NormalizedWindspeed)
  wsdata
}

unnormalize = function(data, time, refdata, m = 0.5){
  library(stringr)
  str = str_split_fixed(refdata$Time," ",2)
  colnames(str) <- c("Date","Time")
  wsdata = data.frame(str,refdata[,2:3])
  days = length(unique(wsdata[,1]))
  hour = unique(wsdata[,2])
  V_prime = (wsdata$WindSpeed)^m
  wsdata = data.frame(wsdata,V_prime)
  V_prime_bar = NULL
  Vpred = NULL
  for (i in 1:length(hour)){
    V_prime_bar[i] = mean(wsdata$V_prime[which(wsdata$Time==hour[i])])
  }
  s = NULL
  for (i in 1:length(hour)){
    s[i] = sd(wsdata$V_prime[which(wsdata$Time==hour[i])])
  }
  for (i in 1:length(data)){
    Vpred[i] = ((data[i] * s[which(hour == time[i])]) + V_prime_bar[which(hour == time[i])])^(1/m)
  }
  Vpred
}

getmode <- function(data, bin.size = bin.size.default) {
  bin.size.default = "default"
  if (bin.size == "default") {
    h = hist(data, plot = FALSE)
    mode = h$mids[h$counts == max(h$counts)]
    return(mode) 
  }
  else {
    breaks = round((max(wsdata) - min(wsdata))/bin.size,0)
    h = hist(data, breaks = breaks, plot = FALSE)
    mode = h$mids[h$counts == max(h$counts)]
    return(mode)
  }
}

mean.weibull = function (scale, shape){
  scale * (gamma(1+(1/shape)))
}

median.weibull = function (scale, shape){
  scale*((log(2))^(1/shape))
}

mode.weibull = function (scale, shape){
  scale * (((shape-1)/shape)^(1/shape))
}

sd.weibull = function (scale,shape){
  sqrt((scale^2)*(gamma(1+(2/shape)) - ((gamma(1+(1/shape)))^2)))
}

aicc = function (aic, n, p, q){
  aicc = aic + 2*(((p+q+1)^2 + (p+q+1))/(n-p-q))
  aicc
}