################################################################################
#File Description: 
#Function to compute asymmetry in a given dataset

##Takes as input:
##rvec: residual vector
##dirdata: vector of wind direction
##locs: coordinates of wind turbines
##kvalstar: lag at which to compute asymmetry
##per: Period of time
##st: starting index
##end: ending index 
##bands: vector of bandwidths at which to compute asymmetry

##returns as output:
##asym_list: list containing vectors of asymmetries at each lag value
##dist_list: list containing vectors of distances at each lag value
################################################################################

asym.compute.st <- function(rvec,dirdata,locs,kvalstar,per,st,end,bands){
  output.list = list(length = 2)
  #checking direction of wind
  #1 = nothernly , 2 = easterly, 3 = southerly, 4= westerly winds, 0= missing
  dir = as.numeric(dirdata)
  cuts = c(315,45,135,225)
  reff = dir[st:end]
  part = vector(length = length(reff))
  for (i in 1:length(reff)){
    if (is.na(reff[i]) == T){part[i] = 0
    } else{
      if (reff[i] >= cuts[1] & reff[i] <= 360){part[i] = 1}
      if (reff[i] >= 0 & reff[i] < cuts[2]){part[i] = 1}
      if (reff[i] >= cuts[2] & reff[i] < cuts[3]){part[i] = 2}
      if (reff[i] >= cuts[3] & reff[i] < cuts[4]){part[i] = 3}
      if (reff[i] >= cuts[4] & reff[i] < cuts[1]){part[i] = 4}
    }}
  v1 = c(1,2,3,4)
  v2 = c(length(which(part==1)),length(which(part==2)),length(which(part==3)),length(which(part==4)))
  
  #finding sample average of wind direction
  circ.dir = circular(reff,type="angles",units="degrees",rotation="clock")
  av.dir = mean.circular(circ.dir,na.rm=T)
  if (as.numeric(av.dir) < 0){av.dir = as.numeric(av.dir) + 360}
  
  if (av.dir >= cuts[1] & av.dir <= 360){part.id = 1}
  if (av.dir >= 0 & av.dir < cuts[2]){part.id = 1}
  if (av.dir >= cuts[2] & av.dir < cuts[3]){part.id = 2}
  if (av.dir >= cuts[3] & av.dir < cuts[4]){part.id = 3}
  if (av.dir >= cuts[4] & av.dir < cuts[1]){part.id = 4}
  
  #Residual matrix
  Z = matrix(nrow = N,ncol=per ) #matrix with time pers cols and turbines rows
  a = 1
  for (i in 1:per){
    Z[,i] = rvec[a:(a+N-1)] 
    a = a + N
  }
  
  if (part.id == 1 | part.id == 3){dimen = locs$lat}
  if (part.id == 2 | part.id == 4){dimen = locs$long}
  dist_mat = rdist.earth(locs[3:2],miles=F)
  
  asym_list = dist_list = list(length = length(bands))
    k = kvalstar
    vario.mat = matrix(nrow=N,ncol=N)
    
    for (ii in 1:N){
      out = lapply(seq(1,(per-k-1),1), function(j) (Z[ii,j+k] - Z[,j])^2)
      vario.mat[ii,]=sapply(seq(1,N,1),function(jj) (1/(2*(per-k-1)))*sum(sapply(out, `[[`, jj))) #summing the jjth element of each subelement in the list
      flush.console();cat(ii, "of 200","\r")
    }
    
    for (iterbb in 1:length(bands)){
      bandwidth = bands[iterbb]
      LL = av.dir - bandwidth
      UL = av.dir + bandwidth
      
      if (part.id == 1){
        if (LL < 0){case = 1}
        if (LL >= 0){case = 2}
      }
      
    #computing asymmetry
    a = 1
    ddiff = vector(length = 1)
    asym = vector(length = 1)
    
    if (part.id == 1){
      for (i in 1:N){
        for (j in 1:N){
          if (dimen[j] < dimen[i]){ #means that i is north of x
            
            if (locs[i,3] < locs[j,3]){ #i is west of j
              diff.lat = (locs[i,2] - locs[j,2])
              diff.long = -(locs[i,3] - locs[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 180 + 90 + angle}
            
            if (locs[i,3] > locs[j,3]){ #i is east of j
              diff.lat =  (locs[i,2] - locs[j,2])
              diff.long = (locs[i,3] - locs[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle =  90 - angle}
            
            if (case == 1){
              if (angle > 270 & angle < 360){angle = angle - 360}
            }
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(locs[i,3:2],locs[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    
    if (part.id == 2){
      for (i in 1:N){
        for (j in 1:N){
          if (dimen[j] < dimen[i]){ #means that i is east of x
            
            if (locs[i,2] > locs[j,2]){ #i is north of j
              diff.lat = locs[i,2] - locs[j,2]
              diff.long = locs[i,3] - locs[j,3]
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 - angle}
            
            if (locs[i,2] < locs[j,2]){ #i is south of j
              diff.lat =  -(locs[i,2] - locs[j,2])
              diff.long =  (locs[i,3] - locs[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 + angle}
            
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(locs[i,3:2],locs[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    if (part.id == 3){
      for (i in 1:N){
        for (j in 1:N){
          if (dimen[j] > dimen[i]){ #means that i is south of x
            
            if (locs[i,3] < locs[j,3]){ #i is west of j
              diff.lat = -(locs[i,2] - locs[j,2])
              diff.long = -(locs[i,3] - locs[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 180 + 90 - angle}
            
            if (locs[i,3] > locs[j,3]){ #i is east of j
              diff.lat =  -(locs[i,2] - locs[j,2])
              diff.long =  (locs[i,3] - locs[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 90 + angle}
            
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(locs[i,3:2],locs[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    if (part.id == 4){
      for (i in 1:N){
        for (j in 1:N){
          if (dimen[j] > dimen[i]){ #means that i is west of x
            
            if (locs[i,2] > locs[j,2]){ #i is north of j
              diff.lat = locs[i,2] - locs[j,2]
              diff.long = - locs[i,3] + locs[j,3]
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 270 + angle}
            
            if (locs[i,2] < locs[j,2]){ #i is south of j
              diff.lat =  -(locs[i,2] - locs[j,2])
              diff.long =  - (locs[i,3] - locs[j,3])
              angle = atan(diff.lat/diff.long) * 180/pi
              angle = 270 - angle}
            
            
            if (angle >= LL & angle <= UL){
              ddiff[a] = rdist.earth.vec(locs[i,3:2],locs[j,3:2],miles=F)
              asym[a] = vario.mat[i,j] - vario.mat[j,i]
              a = a + 1}}}
      }
    }
    
    asym_corr = asym/as.numeric(var(rvec))
    asym_list[[iterbb]] = asym_corr
    dist_list[[iterbb]] = ddiff
    cat("Iteration number ", iterbb, "out of ", length(bands),"\n")
  }
  #Generating output
  output.list[[1]] = asym_list
  output.list[[2]] = dist_list
  return(output.list)
}