################################################################################
#File Description: 
#Function to fit harmonics to wind speed data

##Takes as input:
##datamat: data matrix with first column corresponding to wind speeds
##         and second column corresponding to time index during the day

##returns as output:
##r:       residual vector
################################################################################

harmonics.fit <- function(datamat){
  cat("FITTING TREND","\n")
  mat = matrix(ncol =5,nrow=nn)
  mat[,1] = rep(1,nn)
  mat[,2] = sin(2*pi*datamat[,2]/24)
  mat[,3] = cos(2*pi*datamat[,2]/24)
  mat[,4] = sin(4*pi*datamat[,2]/24)
  mat[,5] = cos(4*pi*datamat[,2]/24)
  harmonics = function(coefs){
    coeffs = matrix(ncol = 1,nrow=5)
    coeffs[,1] = coefs
    f = mat %*% coeffs
    err = sum((datamat[,1] - f)^2)
    return(err)
  }
  ini = c(mean(datamat[,1],na.rm=T),rep(0,4))
  lse = nlm(harmonics,ini,stepmax=5,steptol=10^-5,iterlim=300)
  beta_vec = matrix(ncol = 1,nrow=5)
  beta_vec[,1] = lse$estimate
  trend = mat %*% beta_vec
  
  #getting residuals
  r = as.numeric(datamat[,1] - trend)

  return(r)
}